==> wls-plugin-3.8.1-sources.jar: com/xebialabs/deployit/plugin/wls/contributor/WlsContainerRestartWithCalculatedOrdersContributor.java <==
package com.xebialabs.deployit.plugin.wls.contributor;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.xebialabs.deployit.plugin.wls.ResourceTargetRestartPolicy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.google.common.base.Predicate;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Iterables;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;

import com.xebialabs.deployit.plugin.api.deployment.planning.Contributor;
import com.xebialabs.deployit.plugin.api.deployment.planning.DeploymentPlanningContext;
import com.xebialabs.deployit.plugin.api.deployment.specification.Delta;
import com.xebialabs.deployit.plugin.api.deployment.specification.Deltas;
import com.xebialabs.deployit.plugin.api.flow.Step;
import com.xebialabs.deployit.plugin.api.reflect.DescriptorRegistry;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import com.xebialabs.deployit.plugin.api.udm.Deployed;
import com.xebialabs.deployit.plugin.wls.container.JmsServer;
import com.xebialabs.deployit.plugin.wls.container.WlsContainer;

import static com.google.common.base.Predicates.and;
import static com.google.common.base.Predicates.not;
import static com.google.common.base.Predicates.or;
import static com.google.common.collect.Iterables.filter;
import static com.google.common.collect.Maps.transformEntries;
import static com.xebialabs.deployit.plugin.api.deployment.specification.Operation.NOOP;
import static com.xebialabs.deployit.plugin.wls.contributor.Predicates2.deltaOf;
import static com.xebialabs.deployit.plugin.wls.contributor.Predicates2.extractDeployed;
import static com.xebialabs.deployit.plugin.wls.contributor.Predicates2.operationIs;

public class WlsContainerRestartWithCalculatedOrdersContributor {

    static final int SHIFT_RESTART_ORDERS = 1;

    @Contributor
    public void restartAfterDeployResourcesOrCopiedArtifact(Deltas deltas, DeploymentPlanningContext result) {

        final Set<Deployed<?, ?>> restartableDeployed = Sets.filter(gatherTargets(deltas.getDeltas()),
                new Predicate<Deployed<?, ?>>() {
                    @Override
                    public boolean apply(Deployed<?, ?> deployed) {
                        final boolean restartTarget = (Boolean) deployed.getProperty("restartTarget");
                        if (deployed.hasProperty("targetRestartPolicy")) {
                            ResourceTargetRestartPolicy rds = deployed.getProperty("targetRestartPolicy");
                            return restartTarget && rds == ResourceTargetRestartPolicy.RESTART;
                        }
                        return restartTarget;
                    }
                });
        logger.debug("restartable deployeds {}", restartableDeployed);

        // Assign the higher order for each targeted containers
        final Map<WlsContainer, Integer> ordersByContainer = Maps.newHashMap();
        for (Deployed<?, ?> target : restartableDeployed) {
            final int createOrder = (Integer) target.getProperty("createOrder");
            final WlsContainer container;
            if (target.getContainer() instanceof JmsServer) {
                JmsServer server = (JmsServer) target.getContainer();
                container = server.getServer();
            }
            else {
                container = (WlsContainer) target.getContainer();
            }

            if (!ordersByContainer.containsKey(container)) {
                ordersByContainer.put(container, 0);
            }
            if (createOrder > ordersByContainer.get(container)) {
                ordersByContainer.put(container, createOrder);
            }
        }

        // Transform the values of the map to list of steps.
        final Collection<List<Step>> steps = transformEntries(ordersByContainer, new Maps.EntryTransformer<WlsContainer, Integer, List<Step>>() {
            @Override
            public List<Step> transformEntry(WlsContainer container, Integer order) {
                logger.debug("create stop/start step for {} --> {} ", container, order);
                final ImmutableList.Builder<Step> builder = new ImmutableList.Builder<Step>();
                final int restartOrder = order + SHIFT_RESTART_ORDERS;
                builder.addAll(container.getStopSteps(restartOrder));
                builder.addAll(container.getStartSteps(restartOrder));
                return builder.build();
            }
        }).values();

        // Add steps to the plan
        for (List<Step> step : steps) {
            result.addSteps(step);
        }
    }

    private static Set<Deployed<?, ?>> gatherTargets(List<Delta> operations) {
        logger.trace("WlsContainerRestartWithCalculatedOrdersContributor.gatherTargets");
        final Type resourcesType = DescriptorRegistry.getDescriptor("wls.Resource").getType();
        final Type copiedArtifactType = DescriptorRegistry.getDescriptor("wls.CopiedArtifact").getType();
        final Iterable<Delta> filter = filter(
                operations,
                and(not(operationIs(NOOP)),
                        or(deltaOf(resourcesType), deltaOf(copiedArtifactType))));
        final Iterable<Deployed<?, ?>> transform = Iterables.transform(filter, extractDeployed());
        final Set<Deployed<?, ?>> deployeds = Sets.newHashSet(transform);
        logger.debug("deployed {}", deployeds);
        return deployeds;
    }

    protected static final Logger logger = LoggerFactory.getLogger(WlsContainerRestartWithCalculatedOrdersContributor.class);

}

==> wls-plugin-4.0.0-sources.jar: com/xebialabs/deployit/plugin/wls/contributor/WlsContainerRestartWithCalculatedOrdersContributor.java <==
package com.xebialabs.deployit.plugin.wls.contributor;

import java.util.Collection;
import java.util.List;
import java.util.Map;
import java.util.Set;

import com.xebialabs.deployit.plugin.wls.ResourceTargetRestartPolicy;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import com.google.common.base.Predicate;
import com.google.common.collect.ImmutableList;
import com.google.common.collect.Iterables;
import com.google.common.collect.Maps;
import com.google.common.collect.Sets;

import com.xebialabs.deployit.plugin.api.deployment.planning.Contributor;
import com.xebialabs.deployit.plugin.api.deployment.planning.DeploymentPlanningContext;
import com.xebialabs.deployit.plugin.api.deployment.specification.Delta;
import com.xebialabs.deployit.plugin.api.deployment.specification.Deltas;
import com.xebialabs.deployit.plugin.api.flow.Step;
import com.xebialabs.deployit.plugin.api.reflect.DescriptorRegistry;
import com.xebialabs.deployit.plugin.api.reflect.Type;
import com.xebialabs.deployit.plugin.api.udm.Deployed;
import com.xebialabs.deployit.plugin.wls.container.JmsServer;
import com.xebialabs.deployit.plugin.wls.container.WlsContainer;

import static com.google.common.base.Predicates.and;
import static com.google.common.base.Predicates.not;
import static com.google.common.base.Predicates.or;
import static com.google.common.collect.Iterables.concat;
import static com.google.common.collect.Iterables.filter;
import static com.google.common.collect.Maps.transformEntries;
import static com.xebialabs.deployit.plugin.api.deployment.specification.Operation.NOOP;
import static com.xebialabs.deployit.plugin.wls.contributor.Predicates2.deltaOf;
import static com.xebialabs.deployit.plugin.wls.contributor.Predicates2.extractDeployed;
import static com.xebialabs.deployit.plugin.wls.contributor.Predicates2.operationIs;

@SuppressWarnings("unused")
public class WlsContainerRestartWithCalculatedOrdersContributor {

    static final int SHIFT_RESTART_ORDERS = 1;

    @Contributor
    public void restartAfterDeployResourcesOrCopiedArtifact(Deltas deltas, DeploymentPlanningContext result) {

        final Set<Deployed<?, ?>> restartableDeployed = Sets.filter(gatherTargets(deltas.getDeltas()),
                new Predicate<Deployed<?, ?>>() {
                    @Override
                    public boolean apply(Deployed<?, ?> deployed) {
                        final boolean restartTarget = (Boolean) deployed.getProperty("restartTarget");
                        if (deployed.hasProperty("targetRestartPolicy")) {
                            ResourceTargetRestartPolicy rds = deployed.getProperty("targetRestartPolicy");
                            return restartTarget && rds == ResourceTargetRestartPolicy.RESTART;
                        }
                        return restartTarget;
                    }
                });
        logger.debug("restartable deployeds {}", restartableDeployed);

        // Assign the higher order for each targeted containers
        final Map<WlsContainer, Integer> ordersByContainer = Maps.newHashMap();
        for (Deployed<?, ?> target : restartableDeployed) {
            final int createOrder = (Integer) target.getProperty("createOrder");
            final WlsContainer container;
            if (target.getContainer() instanceof JmsServer) {
                JmsServer server = (JmsServer) target.getContainer();
                container = server.getServer();
            }
            else {
                container = (WlsContainer) target.getContainer();
            }

            if (!ordersByContainer.containsKey(container)) {
                ordersByContainer.put(container, 0);
            }
            if (createOrder > ordersByContainer.get(container)) {
                ordersByContainer.put(container, createOrder);
            }
        }

        // Transform the values of the map to list of steps.
        final Collection<List<Step>> steps = transformEntries(ordersByContainer, new Maps.EntryTransformer<WlsContainer, Integer, List<Step>>() {
            @Override
            public List<Step> transformEntry(WlsContainer container, Integer order) {
                logger.debug("create stop/start step for {} --> {} ", container, order);
                final ImmutableList.Builder<Step> builder = new ImmutableList.Builder<Step>();
                final int restartOrder = order + SHIFT_RESTART_ORDERS;
                builder.addAll(container.getStopSteps(restartOrder));
                builder.addAll(container.getStartSteps(restartOrder));
                return builder.build();
            }
        }).values();

        // Add steps to the plan
        result.addSteps(concat(steps));
    }

    private static Set<Deployed<?, ?>> gatherTargets(List<Delta> operations) {
        logger.trace("WlsContainerRestartWithCalculatedOrdersContributor.gatherTargets");
        final Type resourcesType = DescriptorRegistry.getDescriptor("wls.Resource").getType();
        final Type copiedArtifactType = DescriptorRegistry.getDescriptor("wls.CopiedArtifact").getType();
        final Iterable<Delta> filter = filter(
                operations,
                and(not(operationIs(NOOP)),
                        or(deltaOf(resourcesType), deltaOf(copiedArtifactType))));
        final Iterable<Deployed<?, ?>> transform = Iterables.transform(filter, extractDeployed());
        final Set<Deployed<?, ?>> deployeds = Sets.newHashSet(transform);
        logger.debug("deployed {}", deployeds);
        return deployeds;
    }

    protected static final Logger logger = LoggerFactory.getLogger(WlsContainerRestartWithCalculatedOrdersContributor.class);

}
